//ZylFileGPSReceiver
//Copyright by Zyl Soft 2003 - 2012

unit ZylFileGPSReceiver;

//{$C-}
interface

{$I IdCompilerDefines.inc}

uses
  ZylCustomGPSReceiver, SysUtils, Windows, Classes
  {$ifndef NODF}
  , Forms
  {$endif}
  {$ifdef RADSTUDIOXE2}
  , Vcl.Forms
  {$endif};


type
  TConnectEvent = procedure(Sender: TObject; const FileName: String) of object;

  TZylFileGPSReceiver = class(TZylCustomGPSReceiver)
  private
    FFileName: String;
    FDelay: Integer;

    FConnected: Boolean;
    FPaused: Boolean;

    FOnConnect: TConnectEvent;
    FOnDisconnect: TConnectEvent;

    procedure SetFileName(const Value: String);
    function ProcessFromFile(const FileName: String): Integer;
  public
    property IsConnected: Boolean read FConnected;
    property IsPaused: Boolean read FPaused;

    //constructor
    constructor Create(AOwner: TComponent);override;
    //destructor
    destructor Destroy();override;

    //starts communication
    procedure Open();
    //stops communication
    procedure Close();
    procedure Pause();
    procedure Continue();

  published
    { Published declarations }
    property FileName: String read FFileName write SetFileName;

    property ForceCheckSum;
    {$ifndef CPPB5}
    property NMEALog;
    property LogDateTime;
    property LogFile;   
    {$endif}
    property Commands;

    property Delay: Integer read FDelay write FDelay default 1000;

    property OnConnect: TConnectEvent read FOnConnect write FOnConnect;
    property OnDisconnect: TConnectEvent read FOnDisconnect write FOnDisconnect;
    //fires when new data was received
    property OnReceive;
    property OnLatitudeChange;
    property OnLongitudeChange;
    property OnAltitudeChange;
    property OnSpeedChange;
    property OnHeadingChange;
    property OnPosition2DChange;
    property OnPosition3DChange;
    property OnSatelliteReceive;
    property OnSatellitesReceive;
    property OnArrival;
    property OnNewCommand;
    property OnUnknownCommand;
    property OnWayPointReceive;
    property OnActiveRouteReceive;    
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Zyl Soft', [TZylFileGPSReceiver]);
end;

{ TZylFileGPSReceiver }

constructor TZylFileGPSReceiver.Create(AOwner: TComponent);
begin
  inherited;

  FFileName := '';
  FDelay := 1000;

  FConnected := False;
  FPaused := False;

  FOnConnect := nil;
  FOnDisconnect := nil;
end;

destructor TZylFileGPSReceiver.Destroy;
begin
  FConnected := False;
  FPaused := False;
  Sleep(50);
  inherited;
end;

procedure TZylFileGPSReceiver.Open;
begin
  if FileExists(FFileName) then
  begin
    ResetPosition;
    ResetActiveRoute;
    ClearWayPoints;
    ResetNavigationInfo;
    ResetReceiverStatus;
    FConnected := True;
    FPaused := False;
    if Assigned(FOnConnect) then
      FOnConnect(Self, FFileName);
    ProcessFromFile(FFileName);
  end
  else
  begin
    raise EZylGPSReceiverException.Create('File does not exist.');
  end;
end;

procedure TZylFileGPSReceiver.Close;
begin
  FConnected := False;
  FPaused := False;
  ResetPosition;
  ResetActiveRoute;
  ClearWayPoints;
  ResetNavigationInfo;
  ResetReceiverStatus;  
end;

function TZylFileGPSReceiver.ProcessFromFile(const FileName: String): Integer;
var
  f: TextFile;
  str: AnsiString;
begin
  Result := 0;
  try
    if FileExists(FileName) then
    begin
      AssignFile(f, FileName);
      Reset(f);
      while FConnected and not Eof(f) do
      begin
        if not FPaused then
        begin
          Readln(f, str);
          Result := Result + 1;
          FeedGPSReceiver(str + #13#10);
          Sleep(FDelay);
        end;
        {$ifndef CPPB5}
        Application.ProcessMessages;
        {$endif}
      end;
      CloseFile(f);
      FConnected := False;
      FPaused := False;
      if Assigned(FOnDisconnect) then
        FOnDisconnect(Self, FFileName);
    end
    else
    begin
      raise EZylGPSReceiverException.Create('File does not exist.');
    end;
  except
  end;  
end;

procedure TZylFileGPSReceiver.SetFileName(const Value: String);
begin
  if (FFileName <> Value) and FileExists(Value) then
  begin
    FFileName := Value;
    if FConnected then
    begin
      Close();
      Sleep(50);
      FPaused := False;
      Open();
    end;
  end;
end;

procedure TZylFileGPSReceiver.Continue;
begin
  FPaused := False;
end;

procedure TZylFileGPSReceiver.Pause;
begin
  FPaused := True;
end;

end.
 